package cn.iocoder.yudao.module.depository.controller.app.warehouse;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.apollo.core.event.QueryChannelInfoEvent;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseService;
import cn.iocoder.yudao.module.depository.vo.dept.DeptWarehouseBaseVO;
import cn.iocoder.yudao.module.depository.vo.dept.DeptWarehouseRequestVo;
import cn.iocoder.yudao.module.depository.vo.warehouse.*;
import cn.iocoder.yudao.module.depository.convert.warehouse.DeptWarehouseConvert;
import cn.iocoder.yudao.module.depository.convert.warehouse.WarehouseConvert;
import cn.iocoder.yudao.module.ecw.dal.dataobject.region.RegionDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.DeptWarehouseDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseLineDO;
import cn.iocoder.yudao.module.ecw.service.region.RegionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import org.springframework.context.ApplicationContext;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "app-web - 仓库")
@RestController
@RequestMapping("/ecw/warehouse")
@Validated
public class AppWarehouseController {

    @Resource
    private WarehouseService warehouseService;
    @Resource
    private ApplicationContext applicationContext;
    @Resource
    private RegionService regionService;

    @GetMapping("/get")
    @ApiOperation("获得仓库")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<WarehouseRespVO> getWarehouse(@RequestParam("id") Long id) {
        WarehouseDO warehouse = warehouseService.getWarehouse(id);
        return success(WarehouseConvert.INSTANCE.convert(warehouse));
    }

    @GetMapping("/list")
    @ApiOperation("获得仓库列表")
    @PreAuthenticated
    @ApiImplicitParam(name = "tradeType", value = "进出口类型,1-进口，2-出口", required = false, example = "1", dataTypeClass = Integer.class)
    public CommonResult<List<WarehouseRespVO>> getWarehouseList(@RequestParam("tradeType") Integer tradeType) {
        List<WarehouseDO> list = warehouseService.getWarehouseList(tradeType);
        return success(WarehouseConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得仓库分页")
    @PreAuthenticated
    public CommonResult<PageResult<WarehouseRespVO>> getWarehousePage(@Valid WarehousePageReqVO pageVO) {
        PageResult<WarehouseDO> pageResult = warehouseService.getWarehousePage(pageVO);
        PageResult<WarehouseRespVO> returnResult = WarehouseConvert.INSTANCE.convertPage(pageResult);
        List<WarehouseRespVO> resultList = returnResult.getList();
        List<Long> warehouseIdList = resultList.stream()
                .map(WarehouseRespVO::getId)
                .collect(Collectors.toList());
        //查找出所有部门
        List<DeptWarehouseDO> deptWarehouseDOList = warehouseService.deptList(warehouseIdList);
        Map<Long, List<Long>> warehouseDeptListMap = new HashMap<>();
        for (DeptWarehouseDO entity : deptWarehouseDOList) {
            List<Long> deptIdList;
            if(warehouseDeptListMap.containsKey(entity.getWarehouseId())) {
                deptIdList = warehouseDeptListMap.get(entity.getWarehouseId());
            } else {
                deptIdList = new ArrayList<>();
            }
            deptIdList.add(entity.getDeptId());
            warehouseDeptListMap.put(entity.getWarehouseId(), deptIdList);
        }
        resultList.forEach(t -> t.setDeptIdList(warehouseDeptListMap.get(t.getId())));
        return success(returnResult);
    }



    @PostMapping("/routerList")
    @ApiOperation("获得仓库路线列表")
    @PreAuthenticated
    public CommonResult<PageResult<WarehouseLineBindRespVO>> getWarehouseRouterList(@Valid  @RequestBody WarehouseLineRequestVO requestVO) {
        List<WarehouseDO> list = warehouseService.getWarehouseRouterList(requestVO);
        List<WarehouseDO> pageList = pagination(list, requestVO.getPageNo(), requestVO.getPageSize());
        long total = 0L;
        if(CollectionUtil.isNotEmpty(list)) {
            total = list.size();
        }
        PageResult<WarehouseDO> pageResult = new PageResult(pageList, total);
        return success(WarehouseConvert.INSTANCE.convertLinePage(pageResult));
    }

    @PostMapping("/adjustmentList")
    @ApiOperation("获得可用调仓列表")
    @ApiImplicitParam(name = "id", value = "当前仓库ID", required = true, dataType = "Long")
    @PreAuthenticated
    public CommonResult<List<WarehouseRespVO>> getAvailableWarehouseAdjustmentList(@RequestParam(value = "id") Long id) {
        List<WarehouseDO> list = warehouseService.getAvailableWarehouseAdjustmentList(id);

        return success(WarehouseConvert.INSTANCE.convertList(list));
    }


    @PostMapping("/changeRouteStatus")
    @ApiOperation("修改路线开通状态")
    @PreAuthenticated
    public CommonResult<Long> changeRouteStatus(@Valid  @RequestBody WarehouseLineBindVO requestVO) {
        Long lineId = warehouseService.changeRouteStatus(requestVO);
        return success(lineId);
    }

    /**
     * 查询已开通的线路
     */
    @PostMapping("/openedRouterList")
    @ApiOperation("获取已开通的线路")
    @PreAuthenticated
    public CommonResult<List<WarehouseLineDO>> openedRouterList(@RequestBody WarehouseLineSearchVO requestVO) {
        if (Objects.nonNull(requestVO.getChannelId())){
            // 如果有渠道信息，则查询可出目的地国家下的目的地城市
            QueryChannelInfoEvent channelInfoEvent = new QueryChannelInfoEvent();
            channelInfoEvent.setChannelId(requestVO.getChannelId());
            applicationContext.publishEvent(channelInfoEvent);
            if (Objects.nonNull(channelInfoEvent.getCountryId()) && channelInfoEvent.getCountryId() > 0L){
                requestVO.setDestCountryId(channelInfoEvent.getCountryId());
            }
        }
        List<WarehouseLineDO> list = warehouseService.openedRouterList(requestVO);
        //过滤掉不显示的线路
        if (CollectionUtil.isNotEmpty(list)) {
            list = list.stream().filter(t -> Boolean.TRUE.equals(t.getIsClientShow())).collect(Collectors.toList());
        }
        return success(list);
    }

    @GetMapping("/quantity/required/{lineId}")
    @ApiOperation("数量是否必填")
    public CommonResult<Boolean> quantityRequired(@PathVariable Long lineId) {
        return success(warehouseService.quantityRequired(lineId));
    }

    /**
     * 查询已开通的线路
     */
    @PostMapping("/openedRouterWarehouseList")
    @ApiOperation("获取已开通线路的仓库")
    @PreAuthenticated
    public CommonResult<List<WarehouseLineDO>> openedRouterWarehouseList(@RequestBody WarehouseLineSearchVO requestVO) {
        List<WarehouseLineDO> list = warehouseService.openedRouterList(requestVO);
        //过滤掉不显示的线路
        if (CollectionUtil.isNotEmpty(list)) {
            list = list.stream().filter(t -> Boolean.TRUE.equals(t.getIsClientShow())).collect(Collectors.toList());
        }
        return success(list);
    }

    @PostMapping("/deptBind")
    @ApiOperation("部门绑定")
    public CommonResult<Set<Long>> deptBind(@RequestBody DeptWarehouseRequestVo requestVo) {
        Set<Long> idList = warehouseService.deptBind(requestVo);
        return success(idList);
    }

    @GetMapping("/getWarehouseTreeList")
    @ApiOperation("获取仓库区域树形列表")
    @ApiImplicitParam(name = "tradeType", value = "进出口类型,1-进口，2-出口", required = false, example = "1", dataTypeClass = Integer.class)
    public  CommonResult<List<WarehouseTreeVO>> getWarehouseTreeList(Integer tradeType){
        List<WarehouseTreeVO> list = warehouseService.getWarehouseTreeList(tradeType);
        return success(list);
    }

    @GetMapping("/getGuojiaAndShiAndWarehouseList")
    @ApiOperation("获取仓库区域并行国家、城市、仓库列表")
    @ApiImplicitParam(name = "tradeType", value = "进出口类型,1-进口，2-出口", required = false, example = "1", dataTypeClass = Integer.class)
    public  CommonResult<WarehouseListVO> getGuojiaAndShiAndWarehouseList(Integer tradeType){
        WarehouseListVO warehouseListVO = warehouseService.getGuojiaAndShiAndWarehouseList(tradeType);
        return success(warehouseListVO);
    }


    @GetMapping("/getRegionList")
    public  CommonResult<List<WarehouseTreeRegionVO>> getRegionList(@RequestParam(value = "type", required = false) Integer type,
            String regionId,String destCountryId,
            String objectiveId ,String destWarehouseId){
        List<WarehouseTreeRegionVO> list =
                warehouseService.getRegionList(
                        type,
                        regionId,
                        destCountryId,
                        objectiveId,
                        destWarehouseId);
        return success(list);
    }


    @PostMapping("/deptList")
    @ApiOperation("部门列表")
    public CommonResult<List<DeptWarehouseBaseVO>> deptList(@RequestBody DeptWarehouseBaseVO requestVo) {
        List<DeptWarehouseDO> entityList = warehouseService.deptList(requestVo);
        return success(DeptWarehouseConvert.INSTANCE.convertList(entityList));
    }

    @PostMapping("/regionWarehouseList")
    @ApiOperation("区域仓库列表")
    public CommonResult<List<RegionWarehouse>> regionWarehouseList() {
        List<RegionDO> list = regionService.getContinentCountry();
        list.sort(Comparator.comparing(RegionDO::getSort));
        List<RegionDO> continentList = list.stream()
                .filter(t -> t.getParentId() == 0)
                .collect(Collectors.toList());
        //剩下的是国家
        list.removeAll(continentList);
        Map<Long, List<RegionDO>> continentCountryMap = list.stream()
                .collect(Collectors.groupingBy(RegionDO::getParentId));

        //查找出国家下的所有仓库
        List<Long> countryIdList = list.stream()
                .map(RegionDO::getId)
                .collect(Collectors.toList());

        List<RegionWarehouse> resultList = new ArrayList<>();

        Map<Long, List<WarehouseDO>> countryWarehouseMap = null;
        if(CollectionUtil.isNotEmpty(countryIdList)) {
            List<WarehouseDO> countryWarehouseList = warehouseService.findByCountryIdList(countryIdList);
            countryWarehouseMap = countryWarehouseList.stream()
                    .collect(Collectors.groupingBy(WarehouseDO::getGuojia));
        }

        for (RegionDO regionDO : continentList) {
            if(!continentCountryMap.containsKey(regionDO.getId())) continue;
            List<RegionDO> countryList = continentCountryMap.get(regionDO.getId());

            RegionWarehouse regionWarehouse = WarehouseConvert.INSTANCE.convert01(regionDO);
            List<RegionWarehouse> children = WarehouseConvert.INSTANCE.convertList03(countryList);
            regionWarehouse.setChildren(children);
            resultList.add(regionWarehouse);

            if(countryWarehouseMap == null) continue;
            for (RegionWarehouse country : children) {
                List<WarehouseDO> countryWarehouseList = countryWarehouseMap.get(country.getId());
                country.setWarehouseList(WarehouseConvert.INSTANCE.convertList(countryWarehouseList));
            }
        }
        return success(resultList);
    }

    private static <T> List<T> pagination(List<T> records, int pageNum, int pageSize) {
        if (CollectionUtil.isEmpty(records)) {
            return null;
        }
        int totalCount = records.size();
        int remainder = totalCount % pageSize;
        int pageCount = (remainder > 0) ? totalCount / pageSize + 1 : totalCount / pageSize;
        if (remainder == 0) {
            return records.stream().skip((pageNum - 1) * pageSize).limit(pageSize).collect(Collectors.toList());
        } else {
            if (pageNum == pageCount) {
                return records.stream().skip((pageNum - 1) * pageSize).limit(totalCount).collect(Collectors.toList());
            } else {
                return records.stream().skip((pageNum - 1) * pageSize).limit(pageSize).collect(Collectors.toList());
            }
        }
    }

}
